//****************************************************************
//* Ce fichier et le fichier lib.h contienent les structures et  * 
//* les fonctions  que vous serez amener à utiliser. Vous n'avez * 
//* normalement pas besoin de modifier ces fichiers.             *
//**************************************************************** 

#include "lib.h"

int w,h;

int arrondi(float x){
  return x+0.5;
}

int abs(int a){
  if(a<0) return -a;
  return a;
}

void sdl_init(){
  SDL_Init(SDL_INIT_VIDEO);
}

Point::Point(){
  x=0;
  y=0;
}

Point::Point(int xx,int yy){
  x=xx;
  y=yy;
}

ostream& operator<<(ostream& os,const Point& P){
  os<<"("<<P.x<<","<<P.y<<")";
  return os;
}

Couleur::Couleur(unsigned char r,unsigned char g,unsigned char b){
  red=r;green=g;blue=b;
}

Couleur::Couleur(unsigned char g){
  red=g;green=g;blue=g;
}

Ecran_SDL::Ecran_SDL(int ww,int hh){
  w=ww;
  h=hh;
  sdl_screen=SDL_SetVideoMode(w, h, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
  rafraichir();
}

void Ecran_SDL::rafraichir(){
  SDL_Flip(sdl_screen);
}


void Ecran_SDL::effacer(){
  SDL_FillRect(sdl_screen,NULL,SDL_MapRGB(sdl_screen->format,0,0,0));
}

void Ecran_SDL::dessine_point(Point P,Couleur c){
  if(P.x>=w-1 or P.x<0 or P.y>=h-1 or P.y<0) return;  
  *((Uint32*)(sdl_screen->pixels)+P.x+(h-1-P.y)*w)=SDL_MapRGB(sdl_screen->format,c.red,c.green,c.blue);
}

void Ecran_SDL::dessine_ligne(Point A,Point B,Couleur c){
  int dx,x,xa,xb,dy,y,ya,yb;
  xa=A.x;
  ya=A.y;
  xb=B.x;
  yb=B.y;
  dx=xb-xa;
  dy=yb-ya;
  if(dx==0){
    if(dy<0) return dessine_ligne(B,A,c);
    for(y=ya;y<=yb;++y){
      dessine_point(Point(xa,y),c);
    }
    return;
  }
  if(abs(dy)<abs(dx)){
    if(dx<0) return dessine_ligne(B,A,c);
    for(x=xa;x<=xb;++x){
      dessine_point(Point(x,((x-xa)*dy)/dx+ya),c);
    }
    return;
  }
  else{
    if(dy<0) return dessine_ligne(B,A,c);
    for(y=ya;y<=yb;++y){
      dessine_point(Point(((y-ya)*dx)/dy+xa,y),c);
    }
    return;
  }
  
}

void Ecran_SDL::dessine_grille(int t,Couleur c){
  int x,y;
  for(x=0;x<w;x+=t){
    for(y=0;y<h;++y){
      dessine_point(Point(x,y),c);
    }
  }
  for(y=0;y<h;y+=t){
    for(x=0;x<w;++x){
      dessine_point(Point(x,y),c);
    }
  }
}

void Ecran_SDL::dessine_point_sur_grille(Point P,int t,Couleur c){
  int x,xmin,xmax,y,ymin,ymax;
  xmin=P.x*t+1;
  xmax=P.x*t+t-1;
  ymin=P.y*t+1;
  ymax=P.y*t+t-1;
  for(x=xmin;x<=xmax;++x){
    for(y=ymin;y<=ymax;++y){
      dessine_point(Point(x,y),c);
    }
  }
}

Evenement attendre_evenement(){
  Evenement e;
  SDL_WaitEvent(&e);
  return e;
}

bool est_quitter(Evenement& e){
  return e.type==SDL_QUIT;
}

bool est_bouton_souris_enfoncer(Evenement& e){
  return e.type==SDL_MOUSEBUTTONDOWN;
}

bool est_bouton_souris_relacher(Evenement& e){
  return e.type==SDL_MOUSEBUTTONUP;
}

bool est_deplacement_souris(Evenement& e){
  return e.type==SDL_MOUSEMOTION;
}

bool est_touche_enfoncer(Evenement& e){
  return e.type==SDL_KEYDOWN;
}

Point lire_position_action_bouton(Evenement& e){
  return Point(e.button.x,h-e.button.y);
}

Point lire_position_souris(Evenement& e){
  return Point(e.motion.x,e.motion.y);
}

SDLKey lire_touche(Evenement& e){
  return e.key.keysym.sym;
}

void deplace_souris(Point P){
  SDL_WarpMouse(P.x,P.y);
}
